function filtParams = CalibrateKalmanSS(filtDat)
% Steady-state Kalman filter implementation using the eigenvector method. 
% This function calibrates the filter by least squares parameter estimation
% from the training data provided in filtDat. It computes the filter gain
% and prior covariance matrix (both steady-state) using the closed-form
% eigenvalue method. The parameters calibrated by this function are used by
% DecodeKalmanSS to execute decoding recursions. 
% If the filter parameters are already known (e.g. due to the physics of 
% the system or available from measurements), they should be simply
% populated into filtParams instead of running this function.
% See [1] for details.
%
% Inputs:
%   filtDat     A structure containing the data used for filter training 
%               (learning the encoding model) and prediction (decoding):
%               encode      Structure containing data for encoding:
%                           X   Kalman filter states data (SxT matrix) 
%                           Y   Kalman filter observations data (MxT matrix) 
%                       where S = no. of states, M = no. of observations,
%                       T = no. of time samples of training data.
% Outputs:
%   filtParams  A structure containing the parameters of the filter learned
%               namely the matrices:
%                           A   state transition matrix
%                           C   observation matrix
%                           Q   state noise covariance matrix
%                           R   observation noise covariance matrix
%                           K   steady-state Kalman filter gain matrix
%                           P   steady-state estimation covariance matrix
%
% Wasim Q. Malik - June 29, 2009 <wqm@mit.edu>
%
%*************************************************************************
% IMPORTANT NOTICE: If you use this code in your work, please cite [1].
%*************************************************************************


%*************************************************************************
%   Set local variables
    Xenc = filtDat.encode.X;    % States
    Yenc = filtDat.encode.Y;    % Observations
    T = size(Xenc, 2);          % No. of time samples

%*************************************************************************
%   Encoding: Learn parameters of linear Gaussian (Kalman) model
    X1 = Xenc(:, 1:end-1);
    X2 = Xenc(:, 2:end);
    A = (X2 * X1') / (X1 * X1');                 % state update coefficient matrix
    C = (Yenc * Xenc') / (Xenc * Xenc');         % state-to-observation relation coefficients
    Q = (X2 - A * X1) * (X2 - A * X1)' / (T - 1);       % state noise covariance
    R = (Yenc - C * Xenc) * (Yenc - C * Xenc)' / T;     % observation noise covariance

%*************************************************************************
%   Calculate the steady-state filter gain
    % Form the Hamiltonian matrix
    h11 = inv(A).';
    h12 = h11 * C.' / R * C;
    h21 = Q * h11;
    h22 = A + Q * h11 * C.' / R * C;
    Ham = [h11 h12; h21 h22];       % Hamiltonian matrix
    [eigVec, eigVal] = eig(Ham);
    eigVal1 = diag(eigVal);
    if any(eigVal1 == 1)
        disp('Error: Got a unit eigenvalue of Hamiltonian matrix in KalmanSSeig_Build');
    end
    indBigEig = eigVal1 > 1;  % Eigenvalues outside unit circle
    eigVec1 = eigVec(:, indBigEig);
    n = size(eigVec1, 2);
    % Steady state matrices
    Pbar_ss = eigVec1(n+1:end, :) / eigVec1(1:n, :);    % Prior estimate of covariance
    K_ss = Pbar_ss * C.' / (C * Pbar_ss * C.' + R);     % Kalman gain
    
%*************************************************************************
%   Store filter parameters
    filtParams.A = A;
    filtParams.C = C;
    filtParams.Q = Q;
    filtParams.R = R;    
    filtParams.K = K_ss;
    filtParams.P = Pbar_ss;

%*************************************************************************
%   References
%   [1] W. Q. Malik, W. Truccolo, E. N. Brown, and L. R. Hochberg, 
%       "Efficient decoding with steady-state Kalman filter in neural
%       interface systems," IEEE Transactions on Neural Systems and
%       Rehabilitation Engineering, vol. 19, no. 1, pp.25-34, Feb. 2011

